// Enforce use of a strict operating context.
'use strict';

// Define handler for this controller.
module.exports = (server) => {

  // Use app dependency container.
  const container = server.container();

  // Register additional dependencies required by this plugin.
  container.register('glob', () => require('glob'));
  container.register('yaml', () => require('js-yaml'));

  // Resolve dependencies for this plugin.
  return container.resolve((config, path, fs, yaml, glob) => {

    // Define methods used internally by this plugin.
    const internals = {};
    internals.loadYamlDoc = (docFile) => {

      return yaml.safeLoad(fs.readFileSync(path.join(process.cwd(), 'node_modules', docFile), 'utf8'));
    };
    internals.loadDefinitionsFile = (docFile) => internals.loadYamlDoc(docFile).definitions;
    internals.loadResponsesFile = (docFile) => internals.loadYamlDoc(docFile).responses;
    internals.getDocumentationFiles = () => {

      return {
        paths: glob.sync('nodewrite-*(core|plugin)-*/docs/routes.yml', { cwd: 'node_modules' }),
        definitions: glob.sync('nodewrite-*(core|plugin)-*/docs/definitions.yml', { cwd: 'node_modules' }),
        responses: glob.sync('nodewrite-*(core|plugin)-*/docs/responses.yml', { cwd: 'node_modules' })
      };
    };

    return {

      index: (request, reply) => {

        // Find documentation.
        const documentation = internals.getDocumentationFiles();
        // API parts.
        const api = config.Core.Docs;
        api.info.host = request.info.host;
        api.paths = documentation.paths.map((docFile) => ({
          plugin: server.getPlugin(docFile),
          yaml: internals.loadYamlDoc(docFile)
        }));
        const paths = {};
        // TODO: Improve this entire for each loop. It's too loopy!
        // Iterate over each of the documented paths.
        api.paths.forEach((swagger) => {
          // Store path's swagger yaml as path object.
          const swaggerPaths = swagger.yaml;
          // Iterate each path.
          for (const route in swaggerPaths) {
            // Store reference to the path's routes.
            const routes = swaggerPaths[route];
            // Iterate over each of the route's opeartions.
            for (const operation in routes) {
              // Does the operation method have assigned opeartionId?
              if (!routes[operation].operationId) {
                // No auto-assign using opeartion method as the operationId.
                routes[operation].operationId = operation;
                // Is it a "get" request with no route params?
                if (operation === 'get' && !route.match(/\{.*\}/)) {
                  // Assume it's a route to "get all" resources.
                  routes[operation].operationId = 'getAll';
                }
              }
              // Tag operation with plugin's namespace.
              routes[operation].tags = [swagger.plugin.namespace];
            }
          }
          // Assign enhanced path object to paths object.
          Object.assign(paths, swaggerPaths);
        });
        api.paths = paths;
        api.responses = documentation.responses.map((docFile) => internals.loadResponsesFile(docFile));
        const responses = {};
        api.responses.forEach((response) => Object.assign(responses, response));
        api.responses = responses;
        api.definitions = documentation.definitions.map((docFile) => internals.loadDefinitionsFile(docFile));
        const definitions = {};
        api.definitions.forEach((definition) => Object.assign(definitions, definition));
        api.definitions = definitions;
        // If forwarded by a local proxy then adjust the base path.
        if (request.headers['x-forwarded-for'] === '127.0.0.1') {
          api.basePath = '/api';
        }
        return reply(api);
      }

    };

  });

};
